@@ -4,16 +4,19 @@
 """Miscellaneous file manipulation functions
 
 """
-from __future__ import print_function, division, unicode_literals, absolute_import
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
 import sys
 import pickle
-import subprocess
+import errno
+import subprocess as sp
 import gzip
 import hashlib
 import locale
 from hashlib import md5
 import os
+import os.path as op
 import re
 import shutil
 import posixpath
@@ -24,14 +27,11 @@ from builtins import str, bytes, open
 
 from .. import logging, config
 from .misc import is_container
-from ..interfaces.traits_extension import isdefined
-
 from future import standard_library
 standard_library.install_aliases()
 
 fmlogger = logging.getLogger('utils')
 
-
 related_filetype_sets = [
     ('.hdr', '.img', '.mat'),
     ('.nii', '.mat'),
@@ -77,8 +77,8 @@ def split_filename(fname):
 
     special_extensions = [".nii.gz", ".tar.gz"]
 
-    pth = os.path.dirname(fname)
-    fname = os.path.basename(fname)
+    pth = op.dirname(fname)
+    fname = op.basename(fname)
 
     ext = None
     for special_ext in special_extensions:
@@ -89,10 +89,11 @@ def split_filename(fname):
             fname = fname[:-ext_len]
             break
     if not ext:
-        fname, ext = os.path.splitext(fname)
+        fname, ext = op.splitext(fname)
 
     return pth, fname, ext
 
+
 def to_str(value):
     """
     Manipulates ordered dicts before they are hashed (Py2/3 compat.)
@@ -104,6 +105,7 @@ def to_str(value):
         retval = to_str_py27(value)
     return retval
 
+
 def to_str_py27(value):
     """
     Encode dictionary for python 2
@@ -121,7 +123,7 @@ def to_str_py27(value):
             venc = to_str_py27(val)
             if venc.startswith(("u'", 'u"')):
                 venc = venc[1:]
-            retval+= entry(kenc, venc)
+            retval += entry(kenc, venc)
         retval += '}'
         return retval
 
@@ -148,6 +150,7 @@ def to_str_py27(value):
         retval = retval[1:]
     return retval
 
+
 def fname_presuffix(fname, prefix='', suffix='', newpath=None, use_ext=True):
     """Manipulates path and name of input filename
 
@@ -174,13 +177,19 @@ def fname_presuffix(fname, prefix='', suffix='', newpath=None, use_ext=True):
     >>> fname_presuffix(fname,'pre','post','/tmp')
     '/tmp/prefoopost.nii.gz'
 
+    >>> from nipype.interfaces.base import Undefined
+    >>> fname_presuffix(fname, 'pre', 'post', Undefined) == fname_presuffix(fname, 'pre', 'post')
+    True
+
     """
     pth, fname, ext = split_filename(fname)
     if not use_ext:
         ext = ''
-    if newpath and isdefined(newpath):
-        pth = os.path.abspath(newpath)
-    return os.path.join(pth, prefix + fname + suffix + ext)
+
+    # No need for isdefined: bool(Undefined) evaluates to False
+    if newpath:
+        pth = op.abspath(newpath)
+    return op.join(pth, prefix + fname + suffix + ext)
 
 
 def fnames_presuffix(fnames, prefix='', suffix='', newpath=None, use_ext=True):
@@ -198,14 +207,14 @@ def hash_rename(filename, hashvalue):
     """
     path, name, ext = split_filename(filename)
     newfilename = ''.join((name, '_0x', hashvalue, ext))
-    return os.path.join(path, newfilename)
+    return op.join(path, newfilename)
 
 
 def check_forhash(filename):
     """checks if file has a hash in its filename"""
     if isinstance(filename, list):
         filename = filename[0]
-    path, name = os.path.split(filename)
+    path, name = op.split(filename)
     if re.search('(_0x[a-z0-9]{32})', name):
         hashvalue = re.findall('(_0x[a-z0-9]{32})', name)
         return True, hashvalue
@@ -216,7 +225,7 @@ def check_forhash(filename):
 def hash_infile(afile, chunk_len=8192, crypto=hashlib.md5):
     """ Computes hash of a file using 'crypto' module"""
     hex = None
-    if os.path.isfile(afile):
+    if op.isfile(afile):
         crypto_obj = crypto()
         with open(afile, 'rb') as fp:
             while True:
@@ -231,7 +240,7 @@ def hash_infile(afile, chunk_len=8192, crypto=hashlib.md5):
 def hash_timestamp(afile):
     """ Computes md5 hash of the timestamp of a file """
     md5hex = None
-    if os.path.isfile(afile):
+    if op.isfile(afile):
         md5obj = md5()
         stat = os.stat(afile)
         md5obj.update(str(stat.st_size).encode())
@@ -250,19 +259,22 @@ def _generate_cifs_table():
     On systems without a ``mount`` command, or with no CIFS mounts, returns an
     empty list.
     """
-    exit_code, output = subprocess.getstatusoutput("mount")
+    exit_code, output = sp.getstatusoutput("mount")
     # Not POSIX
     if exit_code != 0:
         return []
 
     # (path, fstype) tuples, sorted by path length (longest first)
-    mount_info = sorted((line.split()[2:5:2] for line in output.splitlines()),
-                        key=lambda x: len(x[0]),
-                        reverse=True)
+    mount_info = sorted(
+        (line.split()[2:5:2] for line in output.splitlines()),
+        key=lambda x: len(x[0]),
+        reverse=True)
     cifs_paths = [path for path, fstype in mount_info if fstype == 'cifs']
 
-    return [mount for mount in mount_info
-            if any(mount[0].startswith(path) for path in cifs_paths)]
+    return [
+        mount for mount in mount_info
+        if any(mount[0].startswith(path) for path in cifs_paths)
+    ]
 
 
 _cifs_table = _generate_cifs_table()
@@ -288,8 +300,12 @@ def on_cifs(fname):
     return False
 
 
-def copyfile(originalfile, newfile, copy=False, create_new=False,
-             hashmethod=None, use_hardlink=False,
+def copyfile(originalfile,
+             newfile,
+             copy=False,
+             create_new=False,
+             hashmethod=None,
+             use_hardlink=False,
              copy_related_files=True):
     """Copy or link ``originalfile`` to ``newfile``.
 
@@ -325,7 +341,7 @@ def copyfile(originalfile, newfile, copy=False, create_new=False,
     fmlogger.debug(newfile)
 
     if create_new:
-        while os.path.exists(newfile):
+        while op.exists(newfile):
             base, fname, ext = split_filename(newfile)
             s = re.search('_c[0-9]{4,4}$', fname)
             i = 0
@@ -355,9 +371,9 @@ def copyfile(originalfile, newfile, copy=False, create_new=False,
     #       copy of file (same hash)                (keep)
     #       different file (diff hash)              (unlink)
     keep = False
-    if os.path.lexists(newfile):
-        if os.path.islink(newfile):
-            if all((os.readlink(newfile) == os.path.realpath(originalfile),
+    if op.lexists(newfile):
+        if op.islink(newfile):
+            if all((os.readlink(newfile) == op.realpath(originalfile),
                     not use_hardlink, not copy)):
                 keep = True
         elif posixpath.samefile(newfile, originalfile):
@@ -387,7 +403,7 @@ def copyfile(originalfile, newfile, copy=False, create_new=False,
         try:
             fmlogger.debug('Linking File: %s->%s', newfile, originalfile)
             # Use realpath to avoid hardlinking symlinks
-            os.link(os.path.realpath(originalfile), newfile)
+            os.link(op.realpath(originalfile), newfile)
         except OSError:
             use_hardlink = False  # Disable hardlink for associated files
         else:
@@ -414,9 +430,14 @@ def copyfile(originalfile, newfile, copy=False, create_new=False,
         related_file_pairs = (get_related_files(f, include_this_file=False)
                               for f in (originalfile, newfile))
         for alt_ofile, alt_nfile in zip(*related_file_pairs):
-            if os.path.exists(alt_ofile):
-                copyfile(alt_ofile, alt_nfile, copy, hashmethod=hashmethod,
-                         use_hardlink=use_hardlink, copy_related_files=False)
+            if op.exists(alt_ofile):
+                copyfile(
+                    alt_ofile,
+                    alt_nfile,
+                    copy,
+                    hashmethod=hashmethod,
+                    use_hardlink=use_hardlink,
+                    copy_related_files=False)
 
     return newfile
 
@@ -439,7 +460,7 @@ def get_related_files(filename, include_this_file=True):
         if this_type in type_set:
             for related_type in type_set:
                 if include_this_file or related_type != this_type:
-                    related_files.append(os.path.join(path, name + related_type))
+                    related_files.append(op.join(path, name + related_type))
     if not len(related_files):
         related_files = [filename]
     return related_files
@@ -469,8 +490,9 @@ def copyfiles(filelist, dest, copy=False, create_new=False):
     newfiles = []
     for i, f in enumerate(filename_to_list(filelist)):
         if isinstance(f, list):
-            newfiles.insert(i, copyfiles(f, dest, copy=copy,
-                                         create_new=create_new))
+            newfiles.insert(i,
+                            copyfiles(
+                                f, dest, copy=copy, create_new=create_new))
         else:
             if len(outfiles) > 1:
                 destfile = outfiles[i]
@@ -511,9 +533,9 @@ def check_depends(targets, dependencies):
     """
     tgts = filename_to_list(targets)
     deps = filename_to_list(dependencies)
-    return all(map(os.path.exists, tgts)) and \
-        min(map(os.path.getmtime, tgts)) > \
-        max(list(map(os.path.getmtime, deps)) + [0])
+    return all(map(op.exists, tgts)) and \
+        min(map(op.getmtime, tgts)) > \
+        max(list(map(op.getmtime, deps)) + [0])
 
 
 def save_json(filename, data):
@@ -626,12 +648,13 @@ def savepkl(filename, record):
     pickle.dump(record, pkl_file)
     pkl_file.close()
 
+
 rst_levels = ['=', '-', '~', '+']
 
 
 def write_rst_header(header, level=0):
-    return '\n'.join((header, ''.join([rst_levels[level]
-                                       for _ in header]))) + '\n\n'
+    return '\n'.join(
+        (header, ''.join([rst_levels[level] for _ in header]))) + '\n\n'
 
 
 def write_rst_list(items, prefix=''):
@@ -659,7 +682,201 @@ def dist_is_editable(dist):
     # Borrowed from `pip`'s' API
     """
     for path_item in sys.path:
-        egg_link = os.path.join(path_item, dist + '.egg-link')
-        if os.path.isfile(egg_link):
+        egg_link = op.join(path_item, dist + '.egg-link')
+        if op.isfile(egg_link):
             return True
     return False
+
+
+def makedirs(path, exist_ok=False):
+    """
+    Create path, if it doesn't exist.
+
+    Parameters
+    ----------
+    path : output directory to create
+
+    """
+    if not exist_ok:  # The old makedirs
+        os.makedirs(path)
+        return path
+
+    # this odd approach deals with concurrent directory cureation
+    if not op.exists(op.abspath(path)):
+        fmlogger.debug("Creating directory %s", path)
+        try:
+            os.makedirs(path)
+        except OSError:
+            fmlogger.debug("Problem creating directory %s", path)
+            if not op.exists(path):
+                raise OSError('Could not create directory %s' % path)
+    return path
+
+
+def emptydirs(path, noexist_ok=False):
+    """
+    Empty an existing directory, without deleting it. Do not
+    raise error if the path does not exist and noexist_ok is True.
+
+    Parameters
+    ----------
+    path : directory that should be empty
+
+    """
+    fmlogger.debug("Removing contents of %s", path)
+
+    if noexist_ok and not op.exists(path):
+        return True
+
+    if op.isfile(path):
+        raise OSError('path "%s" should be a directory' % path)
+
+    try:
+        shutil.rmtree(path)
+    except OSError as ex:
+        elcont = os.listdir(path)
+        if ex.errno == errno.ENOTEMPTY and not elcont:
+            fmlogger.warning(
+                'An exception was raised trying to remove old %s, but the path '
+                'seems empty. Is it an NFS mount?. Passing the exception.',
+                path)
+        elif ex.errno == errno.ENOTEMPTY and elcont:
+            fmlogger.debug('Folder %s contents (%d items).', path, len(elcont))
+            raise ex
+        else:
+            raise ex
+
+    makedirs(path)
+
+
+def which(cmd, env=None, pathext=None):
+    """
+    Return the path to an executable which would be run if the given
+    cmd was called. If no cmd would be called, return ``None``.
+
+    Code for Python < 3.3 is based on a code snippet from
+    http://orip.org/2009/08/python-checking-if-executable-exists-in.html
+
+    """
+
+    if pathext is None:
+        pathext = os.getenv('PATHEXT', '').split(os.pathsep)
+        pathext.insert(0, '')
+
+    path = os.getenv("PATH", os.defpath)
+    if env and 'PATH' in env:
+        path = env.get("PATH")
+
+    if sys.version_info >= (3, 3):
+        for ext in pathext:
+            filename = shutil.which(cmd + ext, path=path)
+            if filename:
+                return filename
+        return None
+
+    def isexec(path):
+        return os.path.isfile(path) and os.access(path, os.X_OK)
+
+    for ext in pathext:
+        extcmd = cmd + ext
+        fpath, fname = os.path.split(extcmd)
+        if fpath:
+            if isexec(extcmd):
+                return extcmd
+        else:
+            for directory in path.split(os.pathsep):
+                filename = op.join(directory, extcmd)
+                if isexec(filename):
+                    return filename
+    return None
+
+
+def get_dependencies(name, environ):
+    """Return library dependencies of a dynamically linked executable
+
+    Uses otool on darwin, ldd on linux. Currently doesn't support windows.
+
+    """
+    if sys.platform == 'darwin':
+        proc = sp.Popen(
+            'otool -L `which %s`' % name,
+            stdout=sp.PIPE,
+            stderr=sp.PIPE,
+            shell=True,
+            env=environ)
+    elif 'linux' in sys.platform:
+        proc = sp.Popen(
+            'ldd `which %s`' % name,
+            stdout=sp.PIPE,
+            stderr=sp.PIPE,
+            shell=True,
+            env=environ)
+    else:
+        return 'Platform %s not supported' % sys.platform
+    o, e = proc.communicate()
+    return o.rstrip()
+
+
+def canonicalize_env(env):
+    """Windows requires that environment be dicts with bytes as keys and values
+    This function converts any unicode entries for Windows only, returning the
+    dictionary untouched in other environments.
+
+    Parameters
+    ----------
+    env : dict
+        environment dictionary with unicode or bytes keys and values
+
+    Returns
+    -------
+    env : dict
+        Windows: environment dictionary with bytes keys and values
+        Other: untouched input ``env``
+    """
+    if os.name != 'nt':
+        return env
+
+    out_env = {}
+    for key, val in env.items():
+        if not isinstance(key, bytes):
+            key = key.encode('utf-8')
+        if not isinstance(val, bytes):
+            val = val.encode('utf-8')
+        out_env[key] = val
+    return out_env
+
+
+def relpath(path, start=None):
+    """Return a relative version of a path"""
+
+    try:
+        return op.relpath(path, start)
+    except AttributeError:
+        pass
+
+    if start is None:
+        start = os.curdir
+    if not path:
+        raise ValueError("no path specified")
+    start_list = op.abspath(start).split(op.sep)
+    path_list = op.abspath(path).split(op.sep)
+    if start_list[0].lower() != path_list[0].lower():
+        unc_path, rest = op.splitunc(path)
+        unc_start, rest = op.splitunc(start)
+        if bool(unc_path) ^ bool(unc_start):
+            raise ValueError(("Cannot mix UNC and non-UNC paths "
+                              "(%s and %s)") % (path, start))
+        else:
+            raise ValueError("path is on drive %s, start on drive %s" %
+                             (path_list[0], start_list[0]))
+    # Work out how much of the filepath is shared by start and path.
+    for i in range(min(len(start_list), len(path_list))):
+        if start_list[i].lower() != path_list[i].lower():
+            break
+    else:
+        i += 1
+
+    rel_list = [op.pardir] * (len(start_list) - i) + path_list[i:]
+    if not rel_list:
+        return os.curdir
+    return op.join(*rel_list)
